﻿Public Class M2J
    'Structures declarations
    Structure JprStructure          'Necessary JPR data (in the context of this application)
        Public nm As String         'Name of the Chart
        Public sc As Double         'Scale of the Chart
        Public sr As Double         'Resolution of the Chart
        Public it As String         'Type of image of the Chart
        Public ds As Double         'Size of pixel in meters (no a formal JPF variable)
        Public dm As String         'Geographic datum of the Chart
        Public st As Double         'Datum shift (latitude)
        Public sn As Double         'Datum shift (longitude)
        Public pr As String         'Projection of the Chart
        Public zn As String         'UTM zone of the Chart
        Public pp As String         'Central meridian of the Chart
        Public p1 As String         'Latitude of the origin of the Chart
        Public p2 As String         'Scalefactor of the Chart
        Public p3 As String         'False Easting of the Chart
        Public p4 As String         'Fales Northing of the Chart
        Public p5 As String         'First standard parallel of the Chart
        Public p6 As String         'Second standard parallel of the Chart
    End Structure

    Structure Coordinate            'Referenced Pixel and Geographical coordinates
        Public x As String          'Pixel X
        Public y As String          'Pixel Y
        Public lat As String        'Latitude
        Public lon As String        'Longitude
        Public Valid As Integer     'Application internal variable used for validation
    End Structure

    'Variable declarations
    Public MapFilePath As String                    'Filename and path to MAP-file.
    Public MapFile As String                        'Filename MAP-file.
    Public MapPath As String                        'Path to MAP-file.
    Public ChartName As String                      'Filename MAP-file without extention.  (ChartName was used to prevent confusion with the use of map-file)
    Public JprFilePath As String                    'Filename and path to to JPR-file.
    Public MapFileContent As String                 'Content of MAP-file
    Public LineCounter As Integer                   'Amount of lines in MapFileContent (will be (re)calculated for every MAP-file)
    Public LineContent(60) As String                'Content of each line in MapFileContent (will be (re)dimensioned for every MAP-file)
    Public LineField(20) As String                  'Content of each fieldin LineContent 
    Public JprData As JprStructure                  'Variable representing the JPR data
    Public ImageWidth As Integer                    'Width of the image
    Public ImageHeight As Integer                   'Height of teh image
    Public UTM As Boolean                           'Indication map projection is UTM
    Public TM As Boolean                            'Indication map projection is Transvese Mercator
    Public MERC As Boolean                          'Indication map projection is Mercator
    Public LCC As Boolean                           'Indication map projection is Lambert Conformal Conic
    Public ReferencePoint(3) As Coordinate          'Calibration points (topleft, topright, bottomright, bottomleft, valid)
    Public ReferencePointCounter As Integer         'Counted number of reference points in MAP-file
    Public ValidReferencePoints As Integer          'Usable Number of reference points in MAP-file
    Public VertexPoint(4) As Coordinate             'Vertexpoints (topleft, topright, bottomright, bottomleft, valid)
    Public VertexPointNoted As Integer              'Noted number of Vertexpoints in MAP-file 
    Public VertexPointLLCounter As Integer          'Counted number of vertexpoints based on Latitude/longitude
    Public VertexPointXYCounter As Integer          'Counted number of vertexpoints based on X/Y coordinate
    Public ValidVertexPoints As Integer             'Usable number of Vertexpoints in MAP-file 
    Public VertexMax As Integer                     'Maximum number of Vertexpoints in MAP-file 

    Public EndLine As String = Chr(13) + Chr(10)    'Variable representing the end a line of text.

    'Subroutine M2J_Load
    'This sub is will be executed starting the application
    '- It enables and sets the Tooltips for buttons and boxes.
    '- Loading values into DpiBox
    '- It enables/disables buttons

    Private Sub M2J_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        'Declarations
        Dim TipText As New ToolTip()

        'Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        'Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        ' Setting up the TipText text 
        TipText.SetToolTip(Me.FindButton, "Find and select a MAP-file.")
        TipText.SetToolTip(Me.MapInfoBox, "Content of the selected MAP-File.")
        TipText.SetToolTip(Me.ExtractButton, "Extract the data form the MAP-file.")
        TipText.SetToolTip(Me.JprDataBox, "Comments and extracted JPR-data.")
        TipText.SetToolTip(Me.BuildButton, "Build the content for the JPR-file. ")
        TipText.SetToolTip(Me.JprFileBox, "Content for the JPR-file.")
        TipText.SetToolTip(Me.DpiBox, "Select or change the resolution of the chart, 254 dpi (100 dot per cm) is the default.")
        TipText.SetToolTip(Me.SaveButton, "Save the content for JPR-file in the same directory as the MAP-file. If the JPR-file exists, you wil be prompted to over write.")
        TipText.SetToolTip(Me.JprDataBox, "Comments and extracted JPR-data")
        TipText.SetToolTip(Me.DisclaimButton, "Disclaimer for the application 'MAP data to JPR' (M2J).")
        TipText.SetToolTip(Me.ClearButton, "Clear all data with in the applicatation, except the selected resolution.")
        TipText.SetToolTip(Me.EndButton, "Close the application.")

        'Loading DpiBox
        DpiBox.Items.Add("72")
        DpiBox.Items.Add("96")
        DpiBox.Items.Add("100")
        DpiBox.Items.Add("127")
        DpiBox.Items.Add("150")
        DpiBox.Items.Add("200")
        DpiBox.Items.Add("254")
        DpiBox.Items.Add("300")
        DpiBox.Items.Add("508")
        'Default setting
        DpiBox.Text = "254"

        'Initial setting user interface (buttons)
        ExtractButton.Enabled = False
        BuildButton.Enabled = False
        SaveButton.Enabled = False
        DpiBox.Enabled = False

    End Sub

    'Subroutine FindButton_Click
    '- Finding/Selecting and opening a MAP file
    '- Creating values for variables: ChartFileName, MapFile, ChartName, JprFilePath
    '- Breaking content of MAP-file in to lines and  load them in an array (LineContent)
    Private Sub FindButton_Click(sender As Object, e As EventArgs) Handles FindButton.Click
        'Declarations
        Dim FileNameLen, Pointer As Integer
        Dim TempFileName As String
        Dim MapContentLen, ChrPointer, LinePointer As Integer
        Dim TxtChr As String = ""
        Dim FindFile As New OpenFileDialog

        'Clearing all data on screen and variables
        Call ClearButton_Click(sender, e)

        'Setting up dialog
        FindFile.Filter = "Ozi-MAP file (*.map) | *.map"
        FindFile.Multiselect = False
        'Opening dialog
        FindFile.ShowDialog()

        'Retrieve value for MAPfile
        TempFileName = FindFile.FileName

        'Stopping this subroutine if no file was selected (preventing an error)
        '(length TempfileNam = 0)
        If Len(TempFileName) = 0 Then
            MsgBox("No file was selected!", 48, "Warning")
            Exit Sub
        End If

        'Setting values for variables
        MapFilePath = TempFileName
        MapFileValue.Text = TempFileName

        'Retrieving and setting values for variables MapFile and ChartPath
        FileNameLen = Len(TempFileName)
        For Pointer = FileNameLen To 0 Step -1
            If Mid(TempFileName, Pointer, 1) = "\" Then
                MapPath = Mid(TempFileName, 1, Pointer)
                MapFile = Mid(TempFileName, Pointer + 1, FileNameLen - Pointer)
                Exit For
            End If
        Next Pointer

        'Retrieving and setting values for variables ChartName and JprFilePath
        FileNameLen = Len(MapFile)
        For Pointer = FileNameLen To 0 Step -1
            If Mid(MapFile, Pointer, 1) = "." Then
                ChartName = Mid(MapFile, 1, Pointer - 1)
                JprFilePath = MapPath & ChartName & ".jpr"
                Exit For
            End If
        Next Pointer

        'Retreiving content of the MapFile
        MapFileContent = My.Computer.FileSystem.ReadAllText(MapPath + MapFile)

        'Loading MapFileContent into MapInfoBox
        MapInfoBox.Text = MapFileContent

        'Breaking MapFileContent into Lines (in array LineContent)
        '- Determening amount of lines (LineCounter)
        MapContentLen = Len(MapFileContent)
        LineCounter = 1
        For I = 1 To MapContentLen
            If Mid(MapFileContent, I, 1) = Chr(13) Then
                LineCounter = LineCounter + 1
            End If
        Next
        '- Retrieving the lines and put them in the array LineContent
        ReDim LineContent(LineCounter)
        ChrPointer = 1
        LinePointer = 1
        Do While LinePointer < LineCounter And ChrPointer < MapContentLen
            Do
                TxtChr = Mid(MapFileContent, ChrPointer, 1)
                If TxtChr <> Chr(13) And TxtChr <> Chr(10) Then
                    LineContent(LinePointer) = LineContent(LinePointer) + TxtChr
                End If
                ChrPointer = ChrPointer + 1
            Loop Until TxtChr = Chr(13) Or TxtChr = Chr(10)
            If TxtChr = Chr(13) Then LinePointer = LinePointer + 1
        Loop

        'Enabling ExtractButton (to extracting the JPR data)
        ExtractButton.Enabled = True
        ExtractButton.Select()

    End Sub

    'Subroutine ExtractButton_Click
    '- Checking if the MAP-file is a OZI-file
    '- Retrieving all neccesary JPR-data

    Private Sub ExtractButton_Click(sender As Object, e As EventArgs) Handles ExtractButton.Click
        'Declarations
        Dim ChrPointer As Integer
        Dim TmpPointer As Integer
        Dim LinePointer As Integer
        Dim CommentText As String = "== Comment ==" & EndLine
        Dim TmpText As String

        'Checking if MAP-data contain OziExplorer information
        If Mid(LineContent(1), 1, 11) <> "OziExplorer" Then
            JprDataBox.Text = "File " & MapFile & " wasn't created with or for OziExplorer."
            Exit Sub
        End If

        'Setting value ChartName
        JprData.nm = LCase(ChartName)

        'Checking File format of map image by the file extention in line 3
        If Len(LineContent(3)) = 0 Then
            CommentText = CommentText & " file isn't linked to a image file." & EndLine
        Else
            For ChrPointer = Len(LineContent(3)) To 0 Step -1
                If Mid(LineContent(3), ChrPointer, 1) = "." Then
                    JprData.it = LCase(Mid(LineContent(3), ChrPointer + 1, Len(LineContent(3)) - 1))
                    Exit For
                End If
            Next
            Select Case JprData.it
                Case "jpg", "png", "tif", "gif", "bmp"
                    CommentText = CommentText & "Type of corresponding image file (" & JprData.it & ") is valid." & EndLine
                Case Else
                    CommentText = CommentText & "Type of corresponding image file (" & JprData.it & ") isn't valid." & EndLine
            End Select
        End If

        'Splitting line 5 into fields to extract JPR data (datum and datum shift)
        Call SplitLine(LineContent(5))
        JprData.dm = LCase(LineField(1))
        JprData.st = LineField(3)
        JprData.sn = LineField(4)

        'Evaluating and if neccesary translating (MAP > JPR) datum
        Select Case JprData.dm
            Case "wgs 84"
                CommentText = CommentText & "Datum (" & JprData.dm & ") is translated to wgs84." & EndLine
                JprData.dm = "wgs84"
            Case "nad83"
                CommentText = CommentText & "Datum (" & JprData.dm & ") is valid." & EndLine
            Case "nad27"
                CommentText = CommentText & "Datum (" & JprData.dm & ") is valid." & EndLine
            Case "nad27 canada"
                CommentText = CommentText & "Datum (" & JprData.dm & ") is translated to north american 1927 (canada mean)." & EndLine
                JprData.dm = "north american 1927 (canada mean)"
            Case "nad27 conus"
                CommentText = CommentText & "Datum (" & JprData.dm & ") is translated to north american 1927 (continental us)." & EndLine
                JprData.dm = "north american 1927 (continental us)"
            Case Else
                CommentText = CommentText & "Datum (" & JprData.dm & ") isn't valid for M2J." & EndLine
        End Select

        'Splitting line 9 into fields to extract JPR data (projection)
        Call SplitLine(LineContent(9))
        JprData.pr = LCase(LineField(2))

        'Evaluating and if neccesary translating (MAP > JPR) projection
        Select Case JprData.pr
            Case "mercator"
                CommentText = CommentText & "Projection (" & JprData.pr & ") is valid." & EndLine
            Case "transverse mercator"
                CommentText = CommentText & "Projection (" & JprData.pr & ") is valid." & EndLine
            Case "(utm) universal transverse mercator"
                CommentText = CommentText & "Projection (" & JprData.pr & ") is translated to utm." & EndLine
                JprData.pr = "utm"
            Case "lambert conformal conic"
                CommentText = CommentText & "Projection (" & JprData.pr & ") is valid." & EndLine
            Case Else
                CommentText = CommentText & "Projection (" & JprData.pr & ") isn't  valid or isn't supported by M2J." & EndLine
        End Select

        'Inserting UTM evaluation??

        'Splitting line 9 into fields to extract JPR data (projection specific data)
        Call SplitLine(LineContent(40))
        JprData.pp = Trim(LineField(3))     'Central meridian of the Chart
        JprData.p1 = Trim(LineField(2))     'Latitude of the origin of the Chart
        JprData.p2 = Trim(LineField(4))     'Scalefactor of the Chart
        JprData.p3 = Trim(LineField(6))     'False Easting of the Chart
        JprData.p4 = Trim(LineField(5))     'False Northing of the Chart
        JprData.p5 = Trim(LineField(7))     'First standard parallel of the Chart
        JprData.p6 = Trim(LineField(8))     'Second standard parallel of the Chart

        'Evaluation of projections
        '- Resetting variables UTM, TM, MERC and LCC
        UTM = True
        TM = True
        MERC = True
        LCC = True

        '- Checking if all nessecary data ar in place
        If JprData.pp = "" Then LCC = False : TM = False
        If JprData.p1 = "" Then LCC = False : TM = False : MERC = False
        If JprData.p2 = "" Then TM = False
        If JprData.p3 = "" Then TM = False
        If JprData.p4 = "" Then TM = False
        If JprData.p5 = "" Then LCC = False
        If JprData.p6 = "" Then LCC = False

        '- Writing to CommentText
        If JprData.pp = "transverse mercator" And TM = False Then
            CommentText = CommentText & "The data for a transverse mercator projection are incomplete." & EndLine
        End If
        If JprData.pp = "mercator" And MERC = False Then
            CommentText = CommentText & "The data for a mercator projection are incomplete." & EndLine
        End If
        If JprData.pp = "lambert conformal conic" And LCC = False Then
            CommentText = CommentText & "The data for a lambert conformal conic projection are incomplete." & EndLine
        End If

        'Retrieving data for Reference points
        '- Determening number of reference points (RefPoint array)
        ReferencePointCounter = 0
        For LinePointer = 10 To 39
            Call SplitLine(LineContent(LinePointer))
            If Trim(LineField(3)) <> "" Then
                ReferencePointCounter = ReferencePointCounter + 1
            Else
                Exit For
            End If
        Next

        '- Redementioning/resetting RefPoint Array
        ReDim ReferencePoint(ReferencePointCounter)
        For TmpPointer = 1 To ReferencePointCounter
            ReferencePoint(TmpPointer).x = 0
            ReferencePoint(TmpPointer).y = 0
            ReferencePoint(TmpPointer).lat = 0
            ReferencePoint(TmpPointer).lon = 0
            ReferencePoint(TmpPointer).Valid = 0
        Next

        '- Determening values for the Reference points array and number of valid Reference points
        ValidReferencePoints = 0
        For LinePointer = 10 To 9 + ReferencePointCounter
            Call SplitLine(LineContent(LinePointer))
            If Trim(LineField(7)) = "" Or Trim(LineField(10)) = "" Then
                ReferencePoint(LinePointer - 9).Valid = ReferencePoint(LinePointer - 9).Valid + 1
            Else
                ReferencePoint(LinePointer - 9).x = Val(LineField(3))
                ReferencePoint(LinePointer - 9).y = Val(LineField(4))
                ReferencePoint(LinePointer - 9).lat = Val(LineField(7)) + Val(LineField(8)) / 60
                If Trim(LCase(LineField(9))) = "s" Then ReferencePoint(LinePointer - 9).lat = ReferencePoint(LinePointer - 9).lat * -1
                ReferencePoint(LinePointer - 9).lon = Val(LineField(10)) + Val(LineField(11)) / 60
                If Trim(LCase(LineField(12))) = "w" Then ReferencePoint(LinePointer - 9).lon = ReferencePoint(LinePointer - 9).lon * -1
                ValidReferencePoints = ValidReferencePoints + 1
            End If
        Next
        CommentText = CommentText & ValidReferencePoints & " usable reference points were found." & EndLine

        'Retrieving data for the Vertex Polygon
        '- Resetting variables
        VertexPointNoted = 0
        VertexPointLLCounter = 0
        VertexPointXYCounter = 0

        '- Retrieving the number of vertex points
        For LinePointer = 41 To LineCounter
            Call SplitLine(LineContent(LinePointer))
            Select Case LCase(LineField(1))
                Case "mmpnum"
                    VertexPointNoted = Val(LineField(2))
                Case "mmpll"
                    VertexPointLLCounter = VertexPointLLCounter + 1
                Case "mmpxy"
                    VertexPointXYCounter = VertexPointXYCounter + 1
            End Select
        Next

        '- Deteremening size of VertexPoint Array
        VertexMax = VertexPointNoted
        If VertexPointLLCounter > VertexMax Then VertexMax = VertexPointLLCounter
        If VertexPointXYCounter > VertexMax Then VertexMax = VertexPointXYCounter
        If VertexMax <> VertexPointNoted Then
            CommentText = CommentText & "The number of noted vertex points (" & VertexPointNoted & ") is different from number counted (" & VertexMax & ")." & EndLine
        End If

        '- Redementioning/resetting VertexPoint Array
        ReDim VertexPoint(VertexMax)
        For TmpPointer = 1 To VertexMax
            VertexPoint(TmpPointer).x = 0
            VertexPoint(TmpPointer).y = 0
            VertexPoint(TmpPointer).lat = 0
            VertexPoint(TmpPointer).lon = 0
            VertexPoint(TmpPointer).Valid = 0
        Next

        '- Determening values for VertexPoint array and DotSize
        For LinePointer = 41 To LineCounter
            Call SplitLine(LineContent(LinePointer))
            Select Case LCase(LineField(1))
                Case "mmpll"
                    If Trim(LineField(3)) = "" Or Trim(LineField(4)) = "" Then
                        VertexPoint(Val(LineField(2))).Valid = VertexPoint(Val(LineField(2))).Valid + 1
                    Else
                        VertexPoint(Val(LineField(2))).lon = Val(LineField(3))
                        VertexPoint(Val(LineField(2))).lat = Val(LineField(4))
                    End If
                Case "mmpxy"
                    If Trim(LineField(3)) = "" Or Trim(LineField(4)) = "" Then
                        VertexPoint(Val(LineField(2))).Valid = VertexPoint(Val(LineField(2))).Valid + 2
                    Else
                        VertexPoint(Val(LineField(2))).x = Val(LineField(3))
                        VertexPoint(Val(LineField(2))).y = Val(LineField(4))
                    End If
                Case "mm1b"
                    JprData.ds = Val(LineField(2))
            End Select
        Next

        '- Determening number of usable vertex points
        ValidVertexPoints = 0
        For TmpPointer = 1 To VertexMax
            If VertexPoint(TmpPointer).Valid = 0 Then
                ValidVertexPoints = ValidVertexPoints + 1
            End If
        Next
        CommentText = CommentText & ValidVertexPoints & " usable vertex points were found." & EndLine

        'Checking if chart can be calibrated
        If ValidVertexPoints < 3 And ValidReferencePoints < 3 Then
            CommentText = CommentText & "The chart can not calibrated. There aren’t enough points available." & EndLine
        End If

        'Determening UTM zone
        If JprData.pr = "utm" Then
            '- Determening number (for UTM zone)
            Call SplitLine(LineContent(10))
            If Trim(LineField(14)) <> "" Then
                'Based on first reference point
                JprData.zn = Trim(LineField(14))
            Else
                'Based on first vertex point
                JprData.zn = Int((VertexPoint(1).lon + 186) / 6)
                If JprData.zn < 1 Then JprData.zn = JprData.zn + 60
            End If

            'Determening north/south (for UTM zone)
            If ReferencePoint(1).Valid = 1 Then
                'Based on longitude first reference pont
                If ReferencePoint(1).lat >= 0 Then
                    JprData.zn = JprData.zn & "t"
                Else
                    JprData.zn = JprData.zn & "j"
                End If
            Else
                'Based on longitude first vertex pont
                If VertexPoint(1).lat >= 0 Then
                    JprData.zn = JprData.zn & "t"
                Else
                    JprData.zn = JprData.zn & "j"
                End If
            End If
            CommentText = CommentText & "UTM zone is " & JprData.zn & "." & EndLine
        End If

        'Last line of comment
        CommentText = CommentText & "== End comment ==" & EndLine

        'JPR data form MAP file
        CommentText = CommentText & "== Start JPR data from MAP file ==" & EndLine
        '- Genearal data
        CommentText = CommentText & "nm (chartname) = " & JprData.nm & EndLine
        CommentText = CommentText & "it (graphics type) = " & JprData.it & EndLine
        '- Datum data
        CommentText = CommentText & "dm (datum) = " & JprData.dm & EndLine
        CommentText = CommentText & "st (datum shift latitude ) = " & JprData.st & EndLine
        CommentText = CommentText & "sn (datum shift longitude )= " & JprData.sn & EndLine
        '- Projectoion data
        CommentText = CommentText & "pr (projection) = " & JprData.pr & EndLine
        CommentText = CommentText & "pp (central meridian) = " & JprData.pp & EndLine
        CommentText = CommentText & "p1 (latitude of origin) = " & JprData.p1 & EndLine
        CommentText = CommentText & "p2 (scale factor) = " & JprData.p2 & EndLine
        CommentText = CommentText & "p3 (false northing) = " & JprData.p3 & EndLine
        CommentText = CommentText & "p4 (false easting) = " & JprData.p4 & EndLine
        CommentText = CommentText & "p5 (standard parallel 1) = " & JprData.p5 & EndLine
        CommentText = CommentText & "p6 (standard parallel 2) = " & JprData.p6 & EndLine
        CommentText = CommentText & "zn (UTM zone) = " & JprData.zn & EndLine
        '- Scale data
        CommentText = CommentText & "dotsize = " & CommaToDot(JprData.ds) & " meter/dot" & EndLine
        '- Reference points
        For TmpPointer = 1 To ReferencePointCounter
            If ReferencePoint(TmpPointer).Valid = 0 Then
                TmpText = "Usable"
            Else
                TmpText = "Unusable"
            End If
            CommentText = CommentText & "rp" & TmpPointer & " (referencepoint " & TmpPointer & ") = " &
                CommaToDot(ReferencePoint(TmpPointer).lat) & ", " &
                CommaToDot(ReferencePoint(TmpPointer).lon) & ", " &
                ReferencePoint(TmpPointer).x & ", " &
                ReferencePoint(TmpPointer).y & ", " &
            TmpText & EndLine
        Next
        '- Vertex points
        For TmpPointer = 1 To VertexMax
            If VertexPoint(TmpPointer).Valid = 0 Then
                TmpText = "Usable"
            Else
                TmpText = "Unusable"
            End If
            CommentText = CommentText & "vp" & TmpPointer & " (vertexpoint " & TmpPointer & ") = " &
                CommaToDot(VertexPoint(TmpPointer).lat) & ", " &
                CommaToDot(VertexPoint(TmpPointer).lon) & ", " &
                VertexPoint(TmpPointer).x & ", " &
                VertexPoint(TmpPointer).y & ", " &
            TmpText & EndLine
        Next
        CommentText = CommentText & "== End JPR data from MAP file ==" & EndLine

        'Copying Comment to JprDataBox
        JprDataBox.Text = CommentText

        'Enabling BuildButton
        BuildButton.Enabled = True
        BuildButton.Select()

    End Sub

    'Subroutine BuildButton_Click builds the correct content for the JPR-file
    Private Sub BuildButton_Click(sender As Object, e As EventArgs) Handles BuildButton.Click

        'Building Content for JPR-File bij using function BuildJprContent
        JprFileBox.Text = BuildJprContent()

        'Enabling SaveButton
        SaveButton.Enabled = True
        SaveButton.Select()
        DpiBox.Enabled = True

    End Sub

    'Subroutine SaveButton_Click 
    '-  Saving the JrpFileBox.text to a JPR-file in the same directory 
    '   as the orginal MAP file. If this JPR file exists a warning will be prompted.
    Private Sub SaveButton_Click(sender As Object, e As EventArgs) Handles SaveButton.Click
        'Declarations
        Dim Answer As Integer       'Used as answer value for MsgBox

        'Checking for an existing file
        If My.Computer.FileSystem.FileExists(JprFilePath) = True Then
            '(Msgbox appearance: 4 = Yes/No, 32 = Question mark, 256 = second button default, 292 = total)
            Answer = MsgBox("JPR-file allready exits. Overwrite?", 292, "Warning")
            If Answer = 6 Then
                'Overwriting existing JPR-file if answer on MsgBox = Yes (Answer = 6)
                My.Computer.FileSystem.WriteAllText(JprFilePath, JprFileBox.Text, False)
            End If
        Else
            'Creating JPR-file
            My.Computer.FileSystem.WriteAllText(JprFilePath, JprFileBox.Text, False)
        End If

    End Sub

    'Subroutne DisclaimButton shows the disclaimer for hzns.nl
    Private Sub DisclaimButton_Click(sender As Object, e As EventArgs) Handles DisclaimButton.Click
        'Declarations
        Dim MsgText As String

        'Disclaimer message
        MsgText = "This application is provided 'As is'. The use of the application is on your own risk. " &
            "Direct or indirect damage by using this application is users responsibility, " &
            "not the application-builders. Redistribution by a third party (commercial of non-commercial) " &
            "is prohibit. Download the application direct from www.hzns.nl."
        MsgBox(MsgText, 48, "Disclaimer")

    End Sub

    'Subroutine ClearButton_Click
    '- Clearing all data from screen
    '- disabling CreateButton and SaveButton
    Private Sub ClearButton_Click(sender As Object, e As EventArgs) Handles ClearButton.Click
        'Declarations
        Dim Pointer As Integer

        'Clearing textboxes and lables
        MapInfoBox.Text = ""
        JprDataBox.Text = ""
        JprFileBox.Text = ""
        MapFileValue.Text = ""

        'Disabling buttons and DpiBox
        ExtractButton.Enabled = False
        BuildButton.Enabled = False
        SaveButton.Enabled = False
        DpiBox.Enabled = False

        'Clearing MAP file Data
        MapFile = ""
        MapPath = ""
        MapFilePath = ""
        ChartName = ""
        JprFilePath = ""
        MapFileContent = ""
        For Pointer = 0 To LineCounter
            LineContent(Pointer) = ""
        Next

        'Focus on FindButton
        FindButton.Select()

    End Sub

    'Unloading and closing application
    Private Sub EndButton_Click(sender As Object, e As EventArgs) Handles EndButton.Click
        Me.Close()
    End Sub

    'Subroutine SplitLine splits a line with comma separeted fields into max 20 fields
    Private Sub SplitLine(LineText As String)
        'Declarations
        Dim FieldPointer, ChrPointer As Integer
        Dim TextChr As String

        'Clear all information Array LineField
        For FieldPointer = 0 To 20
            LineField(FieldPointer) = ""
        Next

        'Splitting line into fields (comma: Chr=44)
        FieldPointer = 1
        ChrPointer = 1
        Do While ChrPointer <= Len(LineText)
            Do
                TextChr = Mid(LineText, ChrPointer, 1)
                If TextChr <> Chr(44) Then
                    LineField(FieldPointer) = LineField(FieldPointer) + TextChr
                End If
                ChrPointer = ChrPointer + 1
            Loop Until TextChr = Chr(44) Or ChrPointer > Len(LineText)
            If TextChr = Chr(44) Then FieldPointer = FieldPointer + 1
        Loop
    End Sub

    'Subroutine DipBox_LostFocus handles the cases LostFocus and SelectedValueChanged (with focus on DpiBox)
    '- recalculating the map scale
    Private Sub DpiBox_LostFocus(sender As Object, e As EventArgs) Handles DpiBox.LostFocus, DpiBox.SelectedValueChanged
        '(only if JprFileContentBox contains information)
        If JprFileBox.Text <> "" Then
            If JprData.sr <> Val(DpiBox.Text) Then
                Call BuildButton_Click(sender, e) '(learned by trail and error)
                SaveButton.Select()
            End If
        End If

    End Sub

    'Subroutine DipBox_PressKey handles the case "Enter"-key was used (with focus on DpiBox)
    '- recalculaing the map scale
    Private Sub DpiBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DpiBox.KeyPress
        If JprFileBox.Text <> "" And e.KeyChar = Chr(13) Then
            If JprData.sr <> Val(DpiBox.Text) Then
                JprData.sr = Val(DpiBox.Text)
                Call BuildButton_Click(sender, e) '(learned by trail and error)
            End If
        End If

    End Sub

    Private Function BuildJprContent()
        'Declarations
        Dim TmpPointer1, TmpPointer2 As Integer
        Dim ContentText As String

        'Adding data line by line to content
        ContentText = "//This file was created using MAP to JPR (M2J) by www.hzns.nl" & EndLine &
            "nm=" & JprData.nm & EndLine &
            "dm=" & JprData.dm & EndLine &
            "st=" & JprData.st & EndLine &
            "sn=" & JprData.sn & EndLine &
            "pr=" & JprData.pr & EndLine

        'Handling projection specific variables
        Select Case JprData.pr
            Case "utm"
                ContentText = ContentText & "zn=" & JprData.zn & EndLine
            Case "mercator"
                ContentText = ContentText & "p1=" & JprData.p1 & EndLine
            Case "transverse mercator"
                ContentText = ContentText & "pp=" & JprData.pp & EndLine &
                    "p1=" & JprData.p1 & EndLine &
                    "p2=" & JprData.p2 & EndLine &
                    "p3=" & JprData.p3 & EndLine &
                    "p4=" & JprData.p4 & EndLine
            Case "lambert conformal conic"
                ContentText = ContentText & "pp=" & JprData.pp & EndLine &
                    "p1=" & JprData.p1 & EndLine &
                    "p5=" & JprData.p5 & EndLine &
                    "p6=" & JprData.p6 & EndLine
            Case Else
                ContentText = ContentText & "//This projection is not supported by MAP To JPR, please check en edit 'p-data'" & EndLine &
                    "pp=" & JprData.pp & EndLine &
                    "p1=" & JprData.p1 & EndLine &
                    "p2=" & JprData.p2 & EndLine &
                    "p3=" & JprData.p3 & EndLine &
                    "p4=" & JprData.p4 & EndLine &
                    "p5=" & JprData.p5 & EndLine &
                    "p6=" & JprData.p6 & EndLine
        End Select

        'Handling scale specific variables
        JprData.sr = Val(DpiBox.Text)
        JprData.sc = Int((JprData.sr / 2.54) * 100 * JprData.ds)
        ContentText = ContentText & "sr=" & JprData.sr & EndLine
        ContentText = ContentText & "sc=1:" & JprData.sc & EndLine

        'Handling scale graphic variables
        ContentText = ContentText & "it=" & JprData.it & EndLine

        'Handling Referrence points
        If ValidVertexPoints < 3 And ValidReferencePoints < 3 Then
            ContentText = ContentText & "//The chart can not calibrated because there aren’t enough points available." & EndLine
        Else
            If ValidVertexPoints > ValidReferencePoints Then
                ContentText = ContentText & "//The vertex polyygon points were use use for calibration." & EndLine
                TmpPointer1 = 0
                For TmpPointer2 = 1 To VertexMax
                    If VertexPoint(TmpPointer2).Valid = 0 Then
                        TmpPointer1 = TmpPointer1 + 1
                        ContentText = ContentText & "vr" & TmpPointer1 & "=" &
                        CommaToDot(VertexPoint(TmpPointer1).lat) & "," &
                        CommaToDot(VertexPoint(TmpPointer1).lon) & "," &
                        VertexPoint(TmpPointer1).x & "," &
                        VertexPoint(TmpPointer1).y & EndLine
                    End If
                Next
            Else
                ContentText = ContentText & "//The reference points were use for calibration." & EndLine
                TmpPointer1 = 0
                For TmpPointer2 = 1 To ReferencePointCounter
                    If ReferencePoint(TmpPointer2).Valid = 0 Then
                        TmpPointer1 = TmpPointer1 + 1
                        ContentText = ContentText & "rp" & TmpPointer1 & "=" &
                        CommaToDot(ReferencePoint(TmpPointer1).lat) & "," &
                        CommaToDot(ReferencePoint(TmpPointer1).lon) & "," &
                        ReferencePoint(TmpPointer1).x & "," &
                        ReferencePoint(TmpPointer1).y & EndLine
                    End If
                Next
            End If
        End If

        'Handling vertex polygon points
        If ValidVertexPoints < 3 Then
            ContentText = ContentText & "//The vertex polygon couldn't established because there aren’t enough points available." & EndLine
        Else
            TmpPointer1 = 0
            For TmpPointer2 = 1 To VertexMax
                If VertexPoint(TmpPointer2).Valid = 0 Then
                    TmpPointer1 = TmpPointer1 + 1
                    ContentText = ContentText & "vp" & TmpPointer1 & "=" &
                    VertexPoint(TmpPointer1).x & "," &
                    VertexPoint(TmpPointer1).y & EndLine
                End If
            Next
        End If

        'Setting return value
        Return ContentText

    End Function

    'Function CommaToDot Converts the numeric expression (for in for example 
    'a Latitude Or Longitude) into string expression with a dot (.) as a seperator
    'And max 6 decimals. (A fale/save construction)
    Private Function CommaToDot(LatLonValue As Double) As String
        'Declarations
        Dim TmpText1, TmpText2 As String
        Dim ChrPointer As Integer
        'Moving character by character form TmpTxt1 to TmpTxt2 
        'unless it is a comma (,). The comma will be replaced by a dot (.).
        TmpText1 = Format(LatLonValue, "0.000000")
        TmpText2 = ""
        For ChrPointer = 1 To Len(TmpText1)
            If Mid(TmpText1, ChrPointer, 1) <> "," Then
                TmpText2 = TmpText2 + Mid(TmpText1, ChrPointer, 1)
            Else
                TmpText2 = TmpText2 + "."
            End If
        Next
        'Returning string to function and ending
        Return TmpText2

    End Function

End Class
